@@ -6,8 +6,12 @@ use File::Spec;
 use Config;
 require 5.004;
 
-use vars qw($VERSION);
-$VERSION = '0.27';
+my $has_version_pm = eval 'use version; 1';
+
+use vars qw($VERSION @ISA $AUTOLOAD);
+# quotes 'version' for 5.004
+$VERSION = eval 'use version; 1' ? 'version'->new('0.32') : '0.32';
+$VERSION = eval $VERSION;
 
 
 =head1 NAME
@@ -48,6 +52,8 @@ Module::Info - Information about Perl modules
   # set/get Module::Info options
   $self->die_on_compilation_error(1);
   my $die_on_error = $mod->die_on_compilation_error;
+  $self->safe(1);
+  my $safe = $mod->safe;
 
 =head1 DESCRIPTION
 
@@ -85,6 +91,8 @@ sub new_from_file {
     $self->{file} = File::Spec->rel2abs($file);
     $self->{dir}  = '';
     $self->{name} = '';
+    $self->{safe} = 0;
+    $self->{use_version} = 0;
 
     return bless $self, $class;
 }
@@ -126,6 +134,8 @@ sub new_from_loaded {
     $module->{name} = $name;
     ($module->{dir} = $filepath) =~ s|/?\Q$mod_file\E$||;
     $module->{dir} = File::Spec->rel2abs($module->{dir});
+    $module->{safe} = 0;
+    $module->{use_version} = 0;
 
     return $module;
 }
@@ -153,7 +163,7 @@ sub _find_all_installed {
 
     @inc = @INC unless @inc;
     my $file = File::Spec->catfile(split /::/, $name) . '.pm';
-    
+
     my @modules = ();
     DIR: foreach my $dir (@inc) {
         # Skip the new code ref in @INC feature.
@@ -168,7 +178,7 @@ sub _find_all_installed {
             last DIR if $find_first_one;
         }
     }
-              
+
     return @modules;
 }
 
@@ -187,7 +197,7 @@ the module.
   my $name = $module->name;
   $module->name($name);
 
-Name of the module (ie. Some::Module).  
+Name of the module (ie. Some::Module).
 
 Module loaded using new_from_file() won't have this information in
 which case you can set it yourself.
@@ -196,7 +206,7 @@ which case you can set it yourself.
 
 sub name {
     my($self) = shift;
-    
+
     $self->{name} = shift if @_;
     return $self->{name};
 }
@@ -216,6 +226,7 @@ sub version {
     local($_, *MOD);
 
     my $parsefile = $self->file;
+    my $safe = $self->safe;
 
     open(MOD, $parsefile) or die $!;
 
@@ -227,22 +238,25 @@ sub version {
 
         chomp;
         next unless /([\$*])(([\w\:\']*)\bVERSION)\b.*\=/;
-        my $eval = qq{
+        my $eval = sprintf qq{
                       package Module::Info::_version;
-                      no strict;
+                      %s
 
                       local $1$2;
                       \$$2=undef; do {
-                          $_
+                          %s
                       }; \$$2
-        };
+        }, ( $safe ? '' : 'no strict;' ), $_;
         local $^W = 0;
-        $result = eval($eval);
-        warn "Could not eval '$eval' in $parsefile: $@" if $@;
+        $result = $self->_eval($eval);
+        warn "Could not eval '$eval' in $parsefile: $@" if $@ && !$safe;
         $result = "undef" unless defined $result;
         last;
     }
     close MOD;
+    $result = 'version'->new($result) # quotes for 5.004
+      if    $self->use_version
+         && (!ref($result) || !UNIVERSAL::isa($result, "version"));
     return $result;
 }
 
@@ -293,7 +307,9 @@ sub is_core {
 
     return scalar grep $self->{dir} eq File::Spec->canonpath($_), 
                            ($Config{installarchlib},
-                            $Config{installprivlib});
+                            $Config{installprivlib},
+                            $Config{archlib},
+                            $Config{privlib});
 }
 
 =back
@@ -422,7 +438,7 @@ sub modules_required {
     push @used_mods, map { /^require not bare (\S+)/; $1 }
                      grep /^require not bare \D/, @mods;
 
-    $used_mods{$_} = [] for @used_mods;
+    foreach ( @used_mods ) { $used_mods{$_} = [] };
     return %used_mods;
 }
 
@@ -477,48 +493,8 @@ sub subroutines {
             grep /at "\Q$mod_file\E" /, @subs;
 }
 
-sub _is_win95() {
-    return $^O eq 'MSWin32' && Win32::GetOSVersion() == 1;
-}
-
-sub _is_macos_classic() {
-    return $^O eq 'MacOS';
-}
-
 sub _get_extra_arguments { '' }
 
-sub _call_perl {
-    my($self, $args) = @_;
-
-    my $perl = _is_macos_classic ? 'perl' : $^X;
-    my $command = "$perl $args";
-    my @out;
-
-    if( _is_win95 ) {
-        require IPC::Open3;
-        local *OUTFH;
-        my($line, $in);
-        my $out = \*OUTFH;
-        my $pid = IPC::Open3::open3($in, $out, $out, $command);
-        close $in;
-        while( defined($line = <OUTFH>) ) {
-            $line =~ s/\r\n$/\n/; # strip CRs
-            push @out, $line;
-        }
-
-        waitpid $pid, 0;
-    }
-    elsif( _is_macos_classic ) {
-        @out = `$command \xb7 Dev:Stdout`;
-    }
-    else {
-        @out = `$command 2>&1`;
-    }
-
-    @out = grep !/^Using.*blib$/, @out;
-    return ($?, @out);
-}
-
 sub _call_B {
     my($self, $arg) = @_;
 
@@ -659,10 +635,10 @@ Module::Info object.
   $module->die_on_compilation_error(1);
   my $flag = $module->die_on_compilation_error;
 
-Sets/gets the "die on compilation error" flag. Whne the flag is off
+Sets/gets the "die on compilation error" flag. When the flag is off
 (default), and a module fails to compile, Module::Info simply emits a
 watning and continues. When the flag is on and a module fails to
-compile, Module::Info die()s with the same error message it would use
+compile, Module::Info C<die()>s with the same error message it would use
 in the warning.
 
 =cut
@@ -674,6 +650,66 @@ sub die_on_compilation_error {
     return $self->{die_on_compilation_error};
 }
 
+=item B<safe>
+
+  $module->safe(0); # default
+  $module->safe(1); # be safer
+  my $flag = $module->safe;
+
+Sets/gets the "safe" flag. When the flag is enabled all operations
+requiring module compilation are forbidden and the C<version()> method
+executes its code in a C<Safe> compartment.
+
+=cut
+
+sub safe {
+    my($self) = shift;
+
+    if( @_ ) {
+        $self->{safe} = $_[0] ? 1 : 0;
+        require Safe if $self->{safe};
+    }
+    return $self->{safe};
+}
+
+sub AUTOLOAD {
+    my($super) = $_[0]->safe ? 'Module::Info::Safe' : 'Module::Info::Unsafe';
+    my($method) = $AUTOLOAD;
+    $method =~ s/^.*::([^:]+)$/$1/;
+
+    return if $method eq 'DESTROY';
+
+    my($code) = $super->can($method);
+
+    die "Can not find method '$method' in Module::Info" unless $code;
+
+    goto &$code;
+}
+
+=item B<use_version>
+
+  $module->use_version(0); # do not use version.pm (default)
+  $module->use_version(1); # use version.pm, die if not present
+  my $flag = $module->use_version;
+
+Sets/gets the "use_version" flag. When the flag is enabled the 'version'
+method always returns a version object.
+
+=cut
+
+sub use_version {
+    my($self) = shift;
+
+    if( @_ ) {
+        die "Can not use 'version.pm' as requested"
+          if $_[0] && !$has_version_pm;
+
+        $self->{use_version} = $_[0] ? 1 : 0;
+    }
+
+    return $self->{use_version};
+}
+
 =back
 
 =head1 AUTHOR
@@ -681,7 +717,7 @@ sub die_on_compilation_error {
 Michael G Schwern <schwern@pobox.com> with code from ExtUtils::MM_Unix,
 Module::InstalledVersion and lots of cargo-culting from B::Deparse.
 
-Mattia Barbon <MBARBON@cpan.org> is the current maintainer.
+Mattia Barbon <mbarbon@cpan.org> is the current maintainer.
 
 =head1 LICENSE
 
@@ -711,4 +747,71 @@ not caching anything.  I'll worry about efficiency later.
 
 =cut
 
+package Module::Info::Safe;
+
+my $root = 'Module::Info::Safe::_safe';
+
+sub _create_compartment {
+    my $safe = Safe->new( $root );
+
+    $safe->permit_only( qw(:base_orig :base_core) );
+
+    return $safe;
+}
+
+sub _eval {
+    my($self, $code) = @_;
+    $self->{compartment} ||= _create_compartment;
+
+    return $self->{compartment}->reval( $code, 0 )
+}
+
+sub _call_perl {
+    die "Module::Info attemped an unsafe operation while in 'safe' mode.";
+}
+
+package Module::Info::Unsafe;
+
+sub _eval { eval($_[1]) }
+
+sub _is_win95() {
+    return $^O eq 'MSWin32' && (Win32::GetOSVersion())[4] == 1;
+}
+
+sub _is_macos_classic() {
+    return $^O eq 'MacOS';
+}
+
+sub _call_perl {
+    my($self, $args) = @_;
+
+    my $perl = _is_macos_classic ? 'perl' : $^X;
+    my $command = "$perl $args";
+    my @out;
+
+    if( _is_win95 ) {
+        require IPC::Open3;
+        local *OUTFH;
+        my($line, $in);
+        my $out = \*OUTFH;
+        my $pid = IPC::Open3::open3($in, $out, $out, $command);
+        close $in;
+        while( defined($line = <OUTFH>) ) {
+            $line =~ s/\r\n$/\n/; # strip CRs
+            push @out, $line;
+        }
+
+        waitpid $pid, 0;
+    }
+    elsif( _is_macos_classic ) {
+        @out = `$command \xb7 Dev:Stdout`;
+    }
+    else {
+        @out = `$command 2>&1`;
+    }
+
+    @out = grep !/^Using.*blib$/, @out;
+    return ($?, @out);
+}
+
 return 'Stepping on toes is what Schwerns do best!  *poing poing poing*';
